
#include "gtk-hotkey-listener.h"
#include "gtk-hotkey-x11-listener.h"
#include "gtk-hotkey-marshal.h"

/* FIXME: The default listener is hardcoded to x11, should be compilation target dependent */

enum  {
	ACTIVATED,
	
	LAST_SIGNAL
};

enum  {
	GTK_HOTKEY_LISTENER_DUMMY_PROPERTY
};

guint						listener_signals[LAST_SIGNAL] = { 0 };

static  gpointer			gtk_hotkey_listener_parent_class = NULL;

static  GtkHotkeyListener	*default_listener = NULL;
static  GType				default_listener_type = G_TYPE_INVALID;
											

/**
 * gtk_hotkey_listener_get_default
 * @returns: A new reference to the default hotkey listener for the platform
 *
 * FIXME: Currently hardcoded to X11
 */
GtkHotkeyListener*
gtk_hotkey_listener_get_default ()
{
	if (default_listener) {
		g_return_val_if_fail (GTK_HOTKEY_IS_LISTENER(default_listener), NULL);
		return g_object_ref (default_listener);
	}
	gtk_hotkey_listener_get_type (); /* This call makes sure the default type ise set */
	g_debug ("Listener Type: %s", g_type_name (default_listener_type));
	
	default_listener = g_object_new (default_listener_type, NULL);
	g_return_val_if_fail (GTK_HOTKEY_IS_LISTENER(default_listener), NULL);
	
	return g_object_ref (default_listener);
}


gboolean
gtk_hotkey_listener_register_hotkey (GtkHotkeyListener  *self,
									 GtkHotkeyInfo		*hotkey,
									 GError				**error)
{
	g_return_val_if_fail (GTK_HOTKEY_IS_LISTENER(self), FALSE);
	
	return GTK_HOTKEY_LISTENER_GET_CLASS (self)->register_hotkey (self, hotkey, error);
}


gboolean
gtk_hotkey_listener_unregister_hotkey (GtkHotkeyListener	*self,
									   GtkHotkeyInfo		*hotkey,
									   GError				**error)
{
	g_return_val_if_fail (GTK_HOTKEY_IS_LISTENER(self), FALSE);
	
	return GTK_HOTKEY_LISTENER_GET_CLASS (self)->unregister_hotkey (self, hotkey, error);
}

/**
 * gtk_hotkey_listener_activated
 * @self: #GtkHotkeyListener to emit the #GtkHotkeyListener::activated signal
 * @hotkey: The #GtkHotkeyInfo the event happened for
 * @event_time: The system time the event happened on. This is useful for
 *              applications to pass through focus stealing prevention when
 *              mapping windows
 *
 * Emit the #GtkHotkeyInfo::activated signal on a hotkey listener.
 */
void
gtk_hotkey_listener_activated (GtkHotkeyListener	*self,
							   GtkHotkeyInfo		*hotkey,
							   guint				event_time)
{
	g_return_if_fail (GTK_HOTKEY_IS_LISTENER(self));
	g_return_if_fail (GTK_HOTKEY_IS_INFO(hotkey));
	
	g_signal_emit (self, listener_signals[ACTIVATED], 0, hotkey, event_time);
}

static void
gtk_hotkey_listener_class_init (GtkHotkeyListenerClass * klass)
{
	gtk_hotkey_listener_parent_class = g_type_class_peek_parent (klass);
	
	/**
	 * GtkHotkeyListener::activated
	 * @listener: The object that emitted the signal
	 * @hotkey: a #GtkHotkeyInfo for the hotkey that was activated
	 * @event_time: Time for event triggering the keypress. This is mainly
	 *              used to pass to window management functions to pass through
	 *              focus stealing prevention
	 *
	 * Emitted when a registered hotkey has been activated.
	 */
	listener_signals[ACTIVATED] = \
	g_signal_new ("activated",
				  GTK_HOTKEY_TYPE_LISTENER,
				  G_SIGNAL_RUN_LAST,
				  0, NULL, NULL,
				  gtk_hotkey_marshal_VOID__OBJECT_UINT,
				  G_TYPE_NONE, 2,
				  GTK_HOTKEY_TYPE_INFO,
				  G_TYPE_UINT);
}


static void
gtk_hotkey_listener_init (GtkHotkeyListener * self)
{
}


GType
gtk_hotkey_listener_get_type (void)
{
	static GType gtk_hotkey_listener_type_id = 0;
	
	if (G_UNLIKELY (gtk_hotkey_listener_type_id == 0)) {
		static const GTypeInfo g_define_type_info = {
			sizeof (GtkHotkeyListenerClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) gtk_hotkey_listener_class_init,
			(GClassFinalizeFunc) NULL,
			NULL,
			sizeof (GtkHotkeyListener),
			0,
			(GInstanceInitFunc) gtk_hotkey_listener_init
		};
		
		gtk_hotkey_listener_type_id = g_type_register_static (G_TYPE_OBJECT,
															  "GtkHotkeyListener",
															  &g_define_type_info,
															  G_TYPE_FLAG_ABSTRACT);
		
		default_listener_type = gtk_hotkey_x11_listener_get_type ();
	}
	return gtk_hotkey_listener_type_id;
}




