
#include "gtk-hotkey-storage.h"
#include "gtk-hotkey-keyfile-storage.h"

enum  {
	GTK_HOTKEY_STORAGE_DUMMY_PROPERTY
};

enum {
	HOTKEY_STORED,
	HOTKEY_DELETED,
	
	LAST_SIGNAL
};

static gpointer		gtk_hotkey_storage_parent_class = NULL;

static GType		default_storage_type = G_TYPE_INVALID;

static GtkHotkeyStorage
					*default_storage = NULL;

guint				storage_signals[LAST_SIGNAL] = { 0 };


GtkHotkeyStorage*
gtk_hotkey_storage_get_default (void)
{	
	if (G_UNLIKELY(default_storage == NULL)) {
		default_storage = GTK_HOTKEY_STORAGE (g_object_new (default_storage_type,
															NULL));
		g_return_val_if_fail (GTK_HOTKEY_IS_STORAGE(default_storage), NULL);
	}
	return g_object_ref(default_storage);
}

GtkHotkeyInfo*
gtk_hotkey_storage_get_hotkey (GtkHotkeyStorage	 *self,
							   const char		*app_id,
							   const char		*key_id,
							   GError			**error)
{
	g_return_val_if_fail (GTK_HOTKEY_IS_STORAGE(self), NULL);
	return GTK_HOTKEY_STORAGE_GET_CLASS (self)->get_hotkey (self, app_id, key_id,
															error);
}

/**
 * gtk_hotkey_storage_get_application_hotkeys
 * @self: The #GtkHotkeyStorage to look hotkeys up in
 * @app_id: Unique application id
 * @error: Place to return a #GError or %NULL
 * @returns: A list of #GtkHotkeyInfo objects. The list should be with freed with
 *           g_list_free() and the hotkey objects should be freed with
 *           g_object_unref().
 * 
 * Look up all hotkeys registered by a given application.
 */
GList*
gtk_hotkey_storage_get_application_hotkeys (GtkHotkeyStorage	*self,
											const char			*app_id,
											GError				**error)
{
	g_return_val_if_fail (GTK_HOTKEY_IS_STORAGE(self), NULL);
	return GTK_HOTKEY_STORAGE_GET_CLASS (self)->get_application_hotkeys (self, app_id, error);
}


GList*
gtk_hotkey_storage_get_all_hotkeys (GtkHotkeyStorage	*self)
{
	g_return_val_if_fail (GTK_HOTKEY_IS_STORAGE(self), NULL);
	return GTK_HOTKEY_STORAGE_GET_CLASS (self)->get_all_hotkeys (self);
}


gboolean
gtk_hotkey_storage_store_hotkey (GtkHotkeyStorage   *self,
								 GtkHotkeyInfo		*info,
								 GError				**error)
{
	g_return_val_if_fail (GTK_HOTKEY_IS_STORAGE(self), FALSE);
	return GTK_HOTKEY_STORAGE_GET_CLASS (self)->store_hotkey (self, info, error);
}

gboolean
gtk_hotkey_storage_delete_hotkey (GtkHotkeyStorage	*self,
								  const gchar		*app_id,
								  const gchar		*key_id,
								  GError			**error)
{
	g_return_val_if_fail (GTK_HOTKEY_IS_STORAGE(self), FALSE);
	return GTK_HOTKEY_STORAGE_GET_CLASS (self)->delete_hotkey (self, app_id,
															   key_id, error);
}

gboolean
gtk_hotkey_storage_has_hotkey (GtkHotkeyStorage		*self,
							   const gchar			*app_id,
							   const gchar			*key_id)
{
	g_return_val_if_fail (GTK_HOTKEY_IS_STORAGE(self), FALSE);
	return GTK_HOTKEY_STORAGE_GET_CLASS (self)->has_hotkey (self, app_id, key_id);
}

void
gtk_hotkey_storage_hotkey_stored (GtkHotkeyStorage	*self,
								 GtkHotkeyInfo		*info)
{
	g_return_if_fail (GTK_HOTKEY_IS_STORAGE(self));
	GTK_HOTKEY_STORAGE_GET_CLASS (self)->hotkey_stored (self, info);
}

void
gtk_hotkey_storage_hotkey_deleted (GtkHotkeyStorage		*self,
								   GtkHotkeyInfo		*info)
{
	g_return_if_fail (GTK_HOTKEY_IS_STORAGE(self));
	GTK_HOTKEY_STORAGE_GET_CLASS (self)->hotkey_deleted (self, info);
}

static void
gtk_hotkey_storage_hotkey_stored_real (GtkHotkeyStorage	*self,
									   GtkHotkeyInfo	*info)
{
	g_return_if_fail (GTK_HOTKEY_IS_INFO(info));
	g_return_if_fail (GTK_HOTKEY_IS_STORAGE(self));
	
	g_signal_emit (self, storage_signals[HOTKEY_STORED], 0, info);
}

static void
gtk_hotkey_storage_hotkey_deleted_real (GtkHotkeyStorage	*self,
										GtkHotkeyInfo		*info)
{
	g_return_if_fail (GTK_HOTKEY_IS_INFO(info));
	g_return_if_fail (GTK_HOTKEY_IS_STORAGE(self));
	
	g_signal_emit (self, storage_signals[HOTKEY_DELETED], 0, info);
}

static void
gtk_hotkey_storage_class_init (GtkHotkeyStorageClass *klass)
{
	gtk_hotkey_storage_parent_class = g_type_class_peek_parent (klass);
	
	klass->hotkey_stored = gtk_hotkey_storage_hotkey_stored_real;
	klass->hotkey_deleted = gtk_hotkey_storage_hotkey_deleted_real;
	
	storage_signals[HOTKEY_STORED] = \
	g_signal_new ("hotkey_stored",
				  GTK_HOTKEY_TYPE_STORAGE,
				  G_SIGNAL_RUN_LAST,
				  0, NULL, NULL,
				  g_cclosure_marshal_VOID__OBJECT,
				  G_TYPE_NONE, 1,
				  G_TYPE_OBJECT);
	
	storage_signals[HOTKEY_DELETED] = \
	g_signal_new ("hotkey_deleted",
				  GTK_HOTKEY_TYPE_STORAGE,
				  G_SIGNAL_RUN_LAST,
				  0, NULL, NULL,
				  g_cclosure_marshal_VOID__OBJECT,
				  G_TYPE_NONE, 1,
				  G_TYPE_OBJECT);
	
}


static void
gtk_hotkey_storage_init (GtkHotkeyStorage * self)
{
	
}


GType
gtk_hotkey_storage_get_type (void)
{
	static GType gtk_hotkey_storage_type_id = 0;
	
	if (G_UNLIKELY (gtk_hotkey_storage_type_id == 0)) {
		static const GTypeInfo g_define_type_info = {
			sizeof (GtkHotkeyStorageClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) gtk_hotkey_storage_class_init,
			(GClassFinalizeFunc) NULL,
			NULL,
			sizeof (GtkHotkeyStorage),
			0,
			(GInstanceInitFunc) gtk_hotkey_storage_init 
		};
		
		gtk_hotkey_storage_type_id = g_type_register_static (G_TYPE_OBJECT, "GtkHotkeyStorage", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		
		default_storage_type = GTK_HOTKEY_TYPE_KEYFILE_STORAGE;
	}
	return gtk_hotkey_storage_type_id;
}
