
#ifndef __GTK_HOTKEY_STORAGE_H__
#define __GTK_HOTKEY_STORAGE_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include "gtk-hotkey-info.h"
#include "gtk-hotkey-error.h"

G_BEGIN_DECLS


#define GTK_HOTKEY_TYPE_STORAGE (gtk_hotkey_storage_get_type ())
#define GTK_HOTKEY_STORAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_HOTKEY_TYPE_STORAGE, GtkHotkeyStorage))
#define GTK_HOTKEY_STORAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_HOTKEY_TYPE_STORAGE, GtkHotkeyStorageClass))
#define GTK_HOTKEY_IS_STORAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_HOTKEY_TYPE_STORAGE))
#define GTK_HOTKEY_IS_STORAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_HOTKEY_TYPE_STORAGE))
#define GTK_HOTKEY_STORAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_HOTKEY_TYPE_STORAGE, GtkHotkeyStorageClass))

typedef struct _GtkHotkeyStorage GtkHotkeyStorage;
typedef struct _GtkHotkeyStorageClass GtkHotkeyStorageClass;
typedef struct _GtkHotkeyStoragePrivate GtkHotkeyStoragePrivate;

struct _GtkHotkeyStorage {
	GObject parent;
	GtkHotkeyStoragePrivate * priv;
};
struct _GtkHotkeyStorageClass {
	GObjectClass	parent;
	GtkHotkeyInfo*  (*get_hotkey)					(GtkHotkeyStorage   *self,
													 const char			*app_id,
													 const char			*key_id,
													 GError				**error);
	GList*			(*get_application_hotkeys)		(GtkHotkeyStorage   *self,
													 const char			*app_id,
													 GError				**error);
	GList*			(*get_all_hotkeys)				(GtkHotkeyStorage   *self);
	gboolean		(*store_hotkey)					(GtkHotkeyStorage   *self,
													 GtkHotkeyInfo		*info,
													 GError				**error);
	gboolean		(*delete_hotkey)				(GtkHotkeyStorage   *self,
													 const gchar		*app_id,
													 const gchar		*key_id,
													 GError				**error);
	gboolean		(*has_hotkey)					(GtkHotkeyStorage   *self,
													 const gchar		*app_id,
													 const gchar		*key_id);
	void			(*hotkey_stored)				(GtkHotkeyStorage   *self,
													 GtkHotkeyInfo		*info);
	void			(*hotkey_deleted)				(GtkHotkeyStorage   *self,
													 GtkHotkeyInfo		*info);
};

GtkHotkeyStorage*		gtk_hotkey_storage_get_default		(void);

GtkHotkeyInfo*			gtk_hotkey_storage_get_hotkey		(GtkHotkeyStorage		*self,
															 const char				*app_id,
															 const char				*key_id,
															 GError					**error);
															 
GList*					gtk_hotkey_storage_get_application_hotkeys
															(GtkHotkeyStorage		*self,
															 const char				*app_id,
															 GError					**error);
															 
GList*					gtk_hotkey_storage_get_all_hotkeys  (GtkHotkeyStorage		*self);

gboolean				gtk_hotkey_storage_store_hotkey		(GtkHotkeyStorage		*self,
															 GtkHotkeyInfo			*info,
															 GError					**error);

gboolean				gtk_hotkey_storage_delete_hotkey	(GtkHotkeyStorage		*self,
															 const gchar			*app_id,
															 const gchar			*key_id,
															 GError					**error);
															 
gboolean				gtk_hotkey_storage_has_hotkey		(GtkHotkeyStorage		*self,
															 const gchar			*app_id,
															 const gchar			*key_id);

void					gtk_hotkey_storage_hotkey_stored	(GtkHotkeyStorage		*self,
															 GtkHotkeyInfo			*hotkey);

void					gtk_hotkey_storage_hotkey_deleted	(GtkHotkeyStorage		*self,
															 GtkHotkeyInfo			*hotkey);

GType					gtk_hotkey_storage_get_type			(void);

G_END_DECLS

#endif
