/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * main.c (winwrangler)
 * Copyright (C) Mikkel Kamstrup Erlandsen 2008 <mikkel.kamstrup@gmail.com>
 * 
 * main.c is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with main.c.  If not, write to:
	 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */

#include "config.h"

#include <gtk/gtk.h>
#include <glib-object.h>
#include <glib.h>
#include <gio/gio.h>

#include "gtk-hotkey-info.h"
#include "gtk-hotkey-listener.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

static gboolean	opt_foo = FALSE;

static GOptionEntry option_entries[] = {
	{"foo", 'f', 0, G_OPTION_ARG_NONE, &opt_foo,
	 "Print 'foo' instead of 'Hello world'"},
	{NULL}
};

static gboolean
do_stuff ()
{
	GError *error;
	GAppInfo		*app;
	GtkHotkeyInfo   *hot;
	GtkHotkeyListener   *listener;
	
	g_warning ("Amm about to do stuffy");
	
	listener = gtk_hotkey_listener_get_default ();
	g_assert (GTK_HOTKEY_IS_LISTENER(listener));
	
	app = G_APP_INFO(g_desktop_app_info_new ("geany.desktop"));
	g_assert (G_IS_APP_INFO(app));
	hot = gtk_hotkey_info_new ("my-app", "my-key", "<Alt>F3", app);
	g_assert (GTK_HOTKEY_IS_INFO(hot));
	
	error = NULL;
	gtk_hotkey_info_register (hot, &error);
	g_signal_connect_swapped (hot, "activated", printf, "W00t - main.c caught key\n");
	if (error)
		g_critical ("Got error: %s", error->message);
	
	return FALSE;
}

int
main (int argc, char *argv[])
{
	GError *error;
	GOptionContext *options;
	
#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif
	
	gtk_init (&argc, &argv);
	
	options = g_option_context_new (NULL);
	g_option_context_add_main_entries (options, option_entries,
									   GETTEXT_PACKAGE);
	g_option_context_add_group (options, gtk_get_option_group (TRUE));
	
	error = NULL;
	if (!g_option_context_parse (options, &argc, &argv, &error))
	{
		g_print ("Bad command line: %s\n", error->message);
		return 1;
	}
	
	if (opt_foo) {
		g_printf ("foo\n");
	} else {
		g_printf ("Hello world\n");
	}
	
	//GtkWidget *win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	//gtk_widget_show_all (win);
	
	g_idle_add (do_stuff, NULL);
	
	GFile *f = g_file_parse_name ("~/");
	g_printf ("File exists: %d", g_file_query_exists(f, NULL));
	
	gtk_main();
	
	return 0;
}
